@@ -5,7 +5,9 @@ api.
 
 from wandb import util
 
-util.vendor_setup()
+reset_path = util.vendor_setup()
 
 from .internal import Api as InternalApi
 from .public import Api as PublicApi
+
+reset_path()
