@@ -28,6 +28,8 @@ import json
 import codecs
 import tempfile
 import sys
+import base64
+import binascii
 from wandb import util
 from wandb.util import has_num
 from wandb.compat import tempfile
@@ -87,10 +89,6 @@ class WBValue(object):
 
     The objects will be serialized as JSON and always have a _type attribute
     that indicates how to interpret the other fields.
-
-    Returns:
-        JSON-friendly `dict` representation of this object that can later be
-            serialized to a string.
     """
 
     _type_mapping = None
@@ -102,19 +100,41 @@ class WBValue(object):
         self._artifact_source = None
 
     def to_json(self, run_or_artifact):
-        """Serializes the object into a JSON blob, optionally using a run or artifact to store additional data."""
+        """Serializes the object into a JSON blob, using a run or artifact to store additional data.
+
+        Args:
+            run_or_artifact (wandb.Run | wandb.Artifact): the Run or Artifact for which this object should be generating
+            JSON for - this is useful to to store additional data if needed.
+
+        Returns:
+            dict: JSON representation
+        """
         raise NotImplementedError
 
     @classmethod
     def from_json(cls, json_obj, source_artifact):
         """Deserialize a `json_obj` into it's class representation. If additional resources were stored in the 
-        `run_or_artifact` artifact during the `to_json` call, then you can expect the `source_artifact` to have
-        the same resources available."""
+        `run_or_artifact` artifact during the `to_json` call, then those resources are expected to be in 
+        the `source_artifact`.
+
+        Args:
+            json_obj (dict): A JSON dictionary to deserialize
+            source_artifact (wandb.Artifact): An artifact which will hold any additional resources which were stored
+            during the `to_json` function.
+        """
         raise NotImplementedError
 
     @classmethod
     def with_suffix(cls, name, filetype="json"):
-        """helper function to return the name with suffix added if needed"""
+        """Helper function to return the name with suffix added if not already
+
+        Args:
+            name (str): the name of the file
+            filetype (str, optional): the filetype to use. Defaults to "json".
+
+        Returns:
+            str: a filename which is suffixed with it's `artifact_type` followed by the filetype
+        """
         if cls.artifact_type is not None:
             suffix = cls.artifact_type + "." + filetype
         else:
@@ -125,7 +145,20 @@ class WBValue(object):
 
     @staticmethod
     def init_from_json(json_obj, source_artifact):
-        """Looks through all subclasses and tries to match the json obj with the class which created it."""
+        """Looks through all subclasses and tries to match the json obj with the class which created it. It will then
+        call that subclass' `from_json` method. Importantly, this function will set the return object's `source_artifact`
+        attribute to the passed in source artifact. This is critical for artifact bookkeeping. If you choose to create
+        a wandb.Value via it's `from_json` method, make sure to properly set this `artifact_source` to avoid data duplication.
+
+        Args:
+            json_obj (dict): A JSON dictionary to deserialize. It must contain a `_type` key. The value of
+            this key is used to lookup the correct subclass to use.
+            source_artifact (wandb.Artifact): An artifact which will hold any additional resources which were stored
+            during the `to_json` function.
+
+        Returns:
+            wandb.Value: a newly created instance of a subclass of wandb.Value
+        """
         class_option = WBValue.type_mapping().get(json_obj["_type"])
         if class_option is not None:
             obj = class_option.from_json(json_obj, source_artifact)
@@ -136,7 +169,11 @@ class WBValue(object):
 
     @staticmethod
     def type_mapping():
-        """Looks through all subclasses and builds a mapping from `artifact_type` to class"""
+        """Returns a map from `artifact_type` to subclass. Used to lookup correct types for deserialization.
+
+        Returns:
+            dict: dictionary of str:class
+        """
         if WBValue._type_mapping is None:
             WBValue._type_mapping = {}
             frontier = [WBValue]
@@ -152,26 +189,30 @@ class WBValue(object):
         return WBValue._type_mapping
 
     def __eq__(self, other):
-        """recommend to override equality comparison to evaluate equality of internal properties"""
         return super(WBValue, self).__eq__(other)
 
     def __ne__(self, other):
-        """optional to override not equal comparison to evaluate equality of internal properties"""
         return not self.__eq__(other)
 
     @property
     def artifact_source(self):
+        """Getter which returns the object's artifact source
+
+        Returns:
+            dict: {"artifact": wandb.Artifact, "name": str} the artifact from which this object was originally
+            stored as well as the name (optional)
+        """
         return self._artifact_source
 
     @artifact_source.setter
     def artifact_source(self, artifact_source):
-        self._artifact_source = {}
+        """Setter for artifact source
 
-        if artifact_source.get("artifact") is not None:
-            self._artifact_source["artifact"] = artifact_source.get("artifact")
-
-        if artifact_source.get("name") is not None:
-            self._artifact_source["name"] = artifact_source.get("name")
+        Args:
+            dict: {"artifact": wandb.Artifact, "name": str} the artifact from which this object was originally
+            stored as well as the name (optional)
+        """
+        self._artifact_source = artifact_source
 
 
 class Histogram(WBValue):
@@ -330,14 +371,18 @@ class Media(WBValue):
             _datatypes_callback(media_path)
 
     def to_json(self, run):
-        """Get the JSON-friendly dict that represents this object.
+        """Serializes the object into a JSON blob, using a run or artifact to store additional data. If `run_or_artifact`
+        is a wandb.Run then `self.bind_to_run()` must have been previously been called.
 
-        Only works if `self.bind_to_run()` has previously been called.
+        Args:
+            run_or_artifact (wandb.Run | wandb.Artifact): the Run or Artifact for which this object should be generating
+            JSON for - this is useful to to store additional data if needed.
 
-        The resulting dict lets you load this object into other W&B runs.
+        Returns:
+            dict: JSON representation
         """
         json_obj = {}
-        wandb_run, _ = _safe_sdk_import()
+        wandb_run, wandb_artifacts = _safe_sdk_import()
         if isinstance(run, wandb_run.Run):
             if not self.is_bound():
                 raise RuntimeError(
@@ -360,7 +405,38 @@ class Media(WBValue):
                     "size": self._size,
                 }
             )
+        elif isinstance(run, wandb_artifacts.Artifact):
+            if self.file_is_set():
+                artifact = run
+                # Checks if the concrete image has already been added to this artifact
+                name = artifact.get_added_local_path_name(self._path)
+                if name is None:
+                    name = os.path.join(
+                        self.get_media_subdir(), os.path.basename(self._path)
+                    )
 
+                    # if not, check to see if there is a source artifact for this object
+                    if (
+                        self.artifact_source is not None
+                        and self.artifact_source["artifact"] != artifact
+                    ):
+                        default_root = self.artifact_source["artifact"]._default_root()
+                        # if there is, get the name of the entry (this might make sense to move to a helper off artifact)
+                        if self._path.startswith(default_root):
+                            name = self._path[len(default_root) :]
+                            name = name.lstrip(os.sep)
+
+                        # Add this image as a reference
+                        path = self.artifact_source["artifact"].get_path(name)
+                        artifact.add_reference(path.ref_url(), name=name)
+                    else:
+                        entry = artifact.add_file(
+                            self._path, name=name, is_tmp=self._is_tmp
+                        )
+                        name = entry.path
+
+                json_obj["path"] = name
+            json_obj["_type"] = self.artifact_type
         return json_obj
 
 
@@ -381,7 +457,7 @@ class BatchableMedia(Media):
 
 
 class Table(Media):
-    """This is a table designed to display small sets of records.
+    """This is a table designed to display sets of records.
 
     Arguments:
         columns ([str]): Names of the columns in the table.
@@ -437,6 +513,7 @@ class Table(Media):
         self.add_data(*row)
 
     def add_data(self, *data):
+        """Add a row of data to the table. Argument length should match column length"""
         if len(data) != len(self.columns):
             raise ValueError(
                 "This table expects {} columns: {}".format(
@@ -1107,6 +1184,11 @@ class Classes(Media):
     artifact_type = "classes"
 
     def __init__(self, class_set):
+        """Classes is holds class metadata intended to be used in concert with other objects when visualizing artifacts
+
+        Args:
+            class_set (list): list of dicts in the form of {"id":int|str, "name":str}
+        """
         super(Classes, self).__init__()
         self._class_set = class_set
         # TODO: validate
@@ -1132,10 +1214,10 @@ class JoinedTable(Media):
     """Joins two tables for visualization in the Artifact UI
 
     Arguments:
-        table1 (str, wandb.Table):
-            the path of a wandb.Table or the table object
+        table1 (str, wandb.Table, ArtifactEntry):
+            the path to a wandb.Table in an artifact, the table object, or ArtifactEntry
         table2 (str, wandb.Table):
-            the path of a wandb.Table or the table object
+            the path to a wandb.Table in an artifact, the table object, or ArtifactEntry
         join_key (str, [str, str]):
             key or keys to perform the join
     """
@@ -1152,14 +1234,14 @@ class JoinedTable(Media):
                 "JoinedTable join_key should be a string or a list of two strings"
             )
 
-        if not isinstance(table1, str) and not isinstance(table1, Table):
+        if not self._validate_table_input(table1):
             raise ValueError(
-                "JoinedTable table1 should be a path or wandb.Table object"
+                "JoinedTable table1 should be an artifact path to a table or wandb.Table object"
             )
 
-        if not isinstance(table2, str) and not isinstance(table2, Table):
+        if not self._validate_table_input(table2):
             raise ValueError(
-                "JoinedTable table2 should be a path or wandb.Table object"
+                "JoinedTable table2 should be an artifact path to a table or wandb.Table object"
             )
 
         self._table1 = table1
@@ -1178,31 +1260,48 @@ class JoinedTable(Media):
 
         return cls(t1, t2, json_obj["join_key"],)
 
+    @staticmethod
+    def _validate_table_input(table):
+        """Helper method to validate that the table input is one of the 3 supported types"""
+        return (
+            (type(table) == str and table.endswith(".table.json"))
+            or isinstance(table, Table)
+            or (hasattr(table, "ref_url") and table.ref_url().endswith(".table.json"))
+        )
+
+    def _ensure_table_in_artifact(self, table, artifact, table_ndx):
+        """Helper method to add the table to the incoming artifact. Returns the path"""
+        if isinstance(table, Table):
+            table_name = "t{}_{}".format(table_ndx, str(id(self)))
+            if (
+                table.artifact_source is not None
+                and table.artifact_source["name"] is not None
+            ):
+                table_name = os.path.basename(table.artifact_source["name"])
+            entry = artifact.add(table, table_name)
+            table = entry.path
+        # Check if this is an ArtifactEntry
+        elif hasattr(table, "ref_url"):
+            # Give the new object a unique, yet deterministic name
+            name = binascii.hexlify(
+                base64.standard_b64decode(table.entry.digest)
+            ).decode("ascii")[:8]
+            entry = artifact.add_reference(
+                table.ref_url(), "{}.table.json".format(name)
+            )[0]
+            table = entry.path
+
+        err_str = "JoinedTable table:{} not found in artifact. Add a table to the artifact using Artifact#add(<table>, {}) before adding this JoinedTable"
+        if table not in artifact._manifest.entries:
+            raise ValueError(err_str.format(table, table))
+
+        return table
+
     def to_json(self, artifact):
         json_obj = super(JoinedTable, self).to_json(artifact)
 
-        table1 = self._table1
-        table2 = self._table2
-
-        if isinstance(self._table1, Table):
-            table_name = "t1_" + str(id(self))
-            if (
-                self._table1.artifact_source is not None
-                and self._table1.artifact_source["name"] is not None
-            ):
-                table_name = os.path.basename(self._table1.artifact_source["name"])
-            entry = artifact.add(self._table1, table_name)
-            table1 = entry.path
-
-        if isinstance(self._table2, Table):
-            table_name = "t2_" + str(id(self))
-            if (
-                self._table2.artifact_source is not None
-                and self._table2.artifact_source["name"] is not None
-            ):
-                table_name = os.path.basename(self._table2.artifact_source["name"])
-            entry = artifact.add(self._table2, table_name)
-            table2 = entry.path
+        table1 = self._ensure_table_in_artifact(self._table1, artifact, 1)
+        table2 = self._ensure_table_in_artifact(self._table2, artifact, 2)
 
         json_obj.update(
             {
@@ -1447,22 +1546,6 @@ class Image(BatchableMedia):
                     "classes must be passed to wandb.Image which have masks or bounding boxes when adding to artifacts"
                 )
 
-            name = artifact.get_added_local_path_name(self._path)
-            if name is None:
-                name = os.path.join(
-                    self.get_media_subdir(), os.path.basename(self._path)
-                )
-                if (
-                    self.artifact_source is not None
-                    and self.artifact_source["artifact"] != artifact
-                ):
-                    path = self.artifact_source["artifact"].get_path(name)
-                    artifact.add_reference(path.ref_url(), name=name)
-                else:
-                    artifact.add_file(self._path, name=name)
-
-            json_dict["path"] = name
-
             if self._classes is not None:
                 # Here, rather than give each class definition it's own name (and entry), we
                 # purposely are giving a non-unique class name of /media/cls.classes.json.
@@ -1559,6 +1642,16 @@ class Image(BatchableMedia):
         width, height = images[0]._image.size
         format = jsons[0]["format"]
 
+        def size_equals_image(image):
+            img_width, img_height = image._image.size
+            return img_width == width and img_height == height
+
+        sizes_match = all(size_equals_image(img) for img in images)
+        if not sizes_match:
+            logging.warning(
+                "Images sizes do not match. This will causes images to be display incorrectly in the UI."
+            )
+
         meta = {
             "_type": "images/separated",
             "width": width,
@@ -1689,6 +1782,28 @@ class BoundingBoxes2D(JSONMetadata):
     artifact_type = "bounding-boxes"
 
     def __init__(self, val, key, **kwargs):
+        """
+        Args:
+            val (dict): dictionary following the form: 
+            {
+                "class_labels": optional mapping from class ids to strings {id: str}
+                "box_data": list of boxes: [
+                    {
+                        "position": {
+                            "minX": float,
+                            "maxX": float,
+                            "minY": float,
+                            "maxY": float,
+                        },
+                        "class_id": 1,
+                        "box_caption": optional str
+                        "scores": optional dict of scores
+                    },
+                    ...
+                ],
+            }
+            key (str): id for set of bounding boxes
+        """
         super(BoundingBoxes2D, self).__init__(val)
         self._val = val["box_data"]
         self._key = key
@@ -1808,6 +1923,20 @@ class ImageMask(Media):
     artifact_type = "mask"
 
     def __init__(self, val, key, **kwargs):
+        """
+        Args:
+            val (dict): dictionary following 1 of two forms: 
+            {
+                "mask_data": 2d array of integers corresponding to classes,
+                "class_labels": optional mapping from class ids to strings {id: str}
+            }
+
+            {
+                "path": path to an image file containing integers corresponding to classes,
+                "class_labels": optional mapping from class ids to strings {id: str}
+            }
+            key (str): id for set of masks
+        """
         super(ImageMask, self).__init__()
 
         if "path" in val:
@@ -1858,39 +1987,15 @@ class ImageMask(Media):
         )
 
     def to_json(self, run_or_artifact):
+        json_dict = super(ImageMask, self).to_json(run_or_artifact)
         wandb_run, wandb_artifacts = _safe_sdk_import()
 
         if isinstance(run_or_artifact, wandb_run.Run):
-            run = run_or_artifact
-            json_dict = super(ImageMask, self).to_json(run)
             json_dict["_type"] = self.type_name()
             return json_dict
         elif isinstance(run_or_artifact, wandb_artifacts.Artifact):
-            artifact = run_or_artifact
-            mask_path = os.path.join(
-                self.get_media_subdir(), os.path.basename(self._path)
-            )
-            mask_name = artifact.get_added_local_path_name(mask_path)
-            mask_entry_digest = None
-            if mask_name is None:
-                if (
-                    self.artifact_source is not None
-                    and self.artifact_source["artifact"] != artifact
-                ):
-                    path = self.artifact_source["artifact"].get_path(mask_path)
-                    mask_entry = artifact.add_reference(path.ref_url(), name=mask_path)[
-                        0
-                    ]
-                else:
-                    mask_entry = artifact.add_file(self._path, name=mask_path)
-
-                mask_name = mask_path
-                mask_entry_digest = mask_entry.digest
-            return {
-                "_type": ImageMask.artifact_type,
-                "path": mask_name,
-                "digest": mask_entry_digest,
-            }
+            # Nothing special to add (used to add "digest", but no longer used.)
+            return json_dict
         else:
             raise ValueError("to_json accepts wandb_run.Run or wandb_artifact.Artifact")
 
