@@ -9,11 +9,13 @@ from wandb.errors.term import termerror
 
 
 RequestUpload = collections.namedtuple(
-    'EventStartUploadJob', ('path', 'save_name', 'artifact_id', 'md5', 'copied',
-                            'save_fn', 'digest'))
+    "EventStartUploadJob",
+    ("path", "save_name", "artifact_id", "md5", "copied", "save_fn", "digest"),
+)
 RequestCommitArtifact = collections.namedtuple(
-    'RequestCommitArtifact', ('artifact_id', 'before_commit', 'on_commit'))
-RequestFinish = collections.namedtuple('RequestFinish', ())
+    "RequestCommitArtifact", ("artifact_id", "before_commit", "on_commit")
+)
+RequestFinish = collections.namedtuple("RequestFinish", ())
 
 
 class StepUpload(object):
@@ -42,7 +44,7 @@ class StepUpload(object):
             if isinstance(event, RequestFinish):
                 break
             self._handle_event(event)
-        
+
         # We've received a finish event. At this point, further Upload requests
         # are invalid. Mark that we're done, which is used to tell the last
         # upload job that it is last.
@@ -67,10 +69,12 @@ class StepUpload(object):
             job.join()
             if job.artifact_id:
                 if event.success:
-                    self._artifacts[job.artifact_id]['pending_count'] -= 1
+                    self._artifacts[job.artifact_id]["pending_count"] -= 1
                     self._maybe_commit_artifact(job.artifact_id)
                 else:
-                    termerror('Uploading artifact file failed. Artifact won\'t be committed.')
+                    termerror(
+                        "Uploading artifact file failed. Artifact won't be committed."
+                    )
             self._running_jobs.pop(job.save_name)
             # If we have any pending jobs, start one now
             if self._pending_jobs:
@@ -79,27 +83,31 @@ class StepUpload(object):
         elif isinstance(event, RequestCommitArtifact):
             if event.artifact_id not in self._artifacts:
                 self._init_artifact(event.artifact_id)
-            self._artifacts[event.artifact_id]['commit_requested'] = True
+            self._artifacts[event.artifact_id]["commit_requested"] = True
             if event.before_commit:
-                self._artifacts[event.artifact_id]['pre_commit_callbacks'].add(event.before_commit)
+                self._artifacts[event.artifact_id]["pre_commit_callbacks"].add(
+                    event.before_commit
+                )
             if event.on_commit:
-                self._artifacts[event.artifact_id]['post_commit_callbacks'].add(event.on_commit)
+                self._artifacts[event.artifact_id]["post_commit_callbacks"].add(
+                    event.on_commit
+                )
             self._maybe_commit_artifact(event.artifact_id)
         elif isinstance(event, RequestUpload):
             if event.artifact_id is not None:
                 if event.artifact_id not in self._artifacts:
                     self._init_artifact(event.artifact_id)
-                self._artifacts[event.artifact_id]['pending_count'] += 1
+                self._artifacts[event.artifact_id]["pending_count"] += 1
             if len(self._running_jobs) == self._max_jobs:
                 self._pending_jobs.append(event)
             else:
                 self._start_upload_job(event)
         else:
-            raise Exception('Programming error: unhandled event: %s' % str(event))
+            raise Exception("Programming error: unhandled event: %s" % str(event))
 
     def _start_upload_job(self, event):
         if not isinstance(event, RequestUpload):
-            raise Exception('Programming error: invalid event')
+            raise Exception("Programming error: invalid event")
 
         # Operations on a single backend file must be serialized. if
         # we're already uploading this file, put the event on the
@@ -110,27 +118,38 @@ class StepUpload(object):
 
         # Start it.
         job = upload_job.UploadJob(
-            self._event_queue, self._stats, self._api,
-            event.save_name, event.path, event.artifact_id, event.md5, event.copied,
-            event.save_fn, event.digest)
+            self._event_queue,
+            self._stats,
+            self._api,
+            event.save_name,
+            event.path,
+            event.artifact_id,
+            event.md5,
+            event.copied,
+            event.save_fn,
+            event.digest,
+        )
         self._running_jobs[event.save_name] = job
         job.start()
 
     def _init_artifact(self, artifact_id):
         self._artifacts[artifact_id] = {
-            'pending_count': 0,
-            'commit_requested': False,
-            'pre_commit_callbacks': set(),
-            'post_commit_callbacks': set(),
+            "pending_count": 0,
+            "commit_requested": False,
+            "pre_commit_callbacks": set(),
+            "post_commit_callbacks": set(),
         }
 
     def _maybe_commit_artifact(self, artifact_id):
         artifact_status = self._artifacts[artifact_id]
-        if artifact_status['pending_count'] == 0 and artifact_status['commit_requested']:
-            for callback in artifact_status['pre_commit_callbacks']:
+        if (
+            artifact_status["pending_count"] == 0
+            and artifact_status["commit_requested"]
+        ):
+            for callback in artifact_status["pre_commit_callbacks"]:
                 callback()
             self._api.commit_artifact(artifact_id)
-            for callback in artifact_status['post_commit_callbacks']:
+            for callback in artifact_status["post_commit_callbacks"]:
                 callback()
 
     def start(self):
