@@ -1,4 +1,5 @@
 from wandb.plot.bar import bar
+from wandb.plot.confusion_matrix import confusion_matrix
 from wandb.plot.histogram import histogram
 from wandb.plot.line import line
 from wandb.plot.pr_curve import pr_curve
@@ -13,4 +14,5 @@ __all__ = [
     "bar",
     "roc_curve",
     "pr_curve",
+    "confusion_matrix",
 ]
