@@ -1,23 +1,20 @@
 #
 # -*- coding: utf-8 -*-
 """
-login.
+Log in to Weights & Biases, authenticating your machine to log data to your
+account.
 """
 
 from __future__ import print_function
 
-import logging
-
 import click
 import wandb
 from wandb.errors.error import UsageError
-from wandb.internal.internal_api import Api
-from wandb.lib import apikey
 
+from .internal.internal_api import Api
+from .lib import apikey
 from .wandb_settings import Settings
 
-logger = logging.getLogger("wandb")
-
 if wandb.TYPE_CHECKING:  # type: ignore
     from typing import Dict, Optional  # noqa: F401 pylint: disable=unused-import
 
@@ -25,21 +22,22 @@ if wandb.TYPE_CHECKING:  # type: ignore
 def login(anonymous=None, key=None, relogin=None, host=None, force=None):
     """Log in to W&B.
 
-    Args:
-        settings (dict, optional): Override settings.
-        relogin (bool, optional): If true, will re-prompt for API key.
-        host (string, optional): The host to connect to
+    Arguments:
         anonymous (string, optional): Can be "must", "allow", or "never".
             If set to "must" we'll always login anonymously, if set to
             "allow" we'll only create an anonymous user if the user
             isn't already logged in.
+        key (string, optional): authentication key.
+        relogin (bool, optional): If true, will re-prompt for API key.
+        host (string, optional): The host to connect to.
+
     Returns:
         bool: if key is configured
 
     Raises:
         UsageError - if api_key can not configured and no tty
     """
-    kwargs = locals()
+    kwargs = dict(locals())
     configured = _login(**kwargs)
     return True if configured else False
 
@@ -61,7 +59,8 @@ class _WandbLogin(object):
         settings_param = kwargs.pop("_settings", None)
         if settings_param:
             login_settings._apply_settings(settings_param)
-        login_settings._apply_login(kwargs)
+        _logger = wandb.setup()._get_logger()
+        login_settings._apply_login(kwargs, _logger=_logger)
 
         # make sure they are applied globally
         self._wl = wandb.setup(settings=login_settings)
@@ -109,7 +108,7 @@ class _WandbLogin(object):
             )
 
     def configure_api_key(self, key):
-        if self._settings._jupyter:
+        if self._settings._jupyter and not self._settings._silent:
             wandb.termwarn(
                 (
                     "If you're specifying your api key in code, ensure this "
@@ -123,12 +122,15 @@ class _WandbLogin(object):
         self._key = key
 
     def update_session(self, key):
+        _logger = wandb.setup()._get_logger()
         settings: Settings = wandb.Settings()
-        settings._apply_source_login(dict(api_key=key))
+        login_settings = dict(api_key=key) if key else dict(mode="offline")
+        settings._apply_source_login(login_settings, _logger=_logger)
         self._wl._update(settings=settings)
         # Whenever the key changes, make sure to pull in user settings
         # from server.
-        self._wl._update_user_settings()
+        if not self._wl.settings._offline:
+            self._wl._update_user_settings()
 
     def prompt_api_key(self):
         api = Api()
@@ -160,11 +162,14 @@ def _login(
     force=None,
     _backend=None,
     _silent=None,
+    _disable_warning=None,
 ):
-    kwargs = locals()
+    kwargs = dict(locals())
+    _disable_warning = kwargs.pop("_disable_warning", None)
 
     if wandb.run is not None:
-        wandb.termwarn("Calling wandb.login() after wandb.init() has no effect.")
+        if not _disable_warning:
+            wandb.termwarn("Calling wandb.login() after wandb.init() has no effect.")
         return True
 
     wlogin = _WandbLogin()
