@@ -21,6 +21,7 @@ import click
 from six import iteritems, string_types
 from six.moves import _thread as thread
 from six.moves.urllib.parse import quote as url_quote
+from six.moves.urllib.parse import urlencode
 import wandb
 from wandb import trigger
 from wandb.apis import internal, public
@@ -39,6 +40,7 @@ from . import wandb_history
 from . import wandb_summary
 from .interface.summary_record import SummaryRecord
 from .lib import (
+    apikey,
     config_util,
     filenames,
     ipython,
@@ -957,19 +959,34 @@ class Run(object):
 
         self._config_callback(data=self._config._as_dict())
 
+    def _get_url_query_string(self):
+        s = self._settings
+
+        # TODO(jhr): migrate to new settings, but for now this is safer
+        api = internal.Api()
+        if api.settings().get("anonymous") != "true":
+            return ""
+
+        api_key = apikey.api_key(settings=s)
+        return "?" + urlencode({"apiKey": api_key})
+
     def _get_project_url(self):
         s = self._settings
         r = self._run_obj
         app_url = wandb.util.app_url(s.base_url)
-        url = "{}/{}/{}".format(app_url, url_quote(r.entity), url_quote(r.project))
+        qs = self._get_url_query_string()
+        url = "{}/{}/{}{}".format(
+            app_url, url_quote(r.entity), url_quote(r.project), qs
+        )
         return url
 
     def _get_run_url(self):
         s = self._settings
         r = self._run_obj
         app_url = wandb.util.app_url(s.base_url)
-        url = "{}/{}/{}/runs/{}".format(
-            app_url, url_quote(r.entity), url_quote(r.project), url_quote(r.run_id)
+        qs = self._get_url_query_string()
+        url = "{}/{}/{}/runs/{}{}".format(
+            app_url, url_quote(r.entity), url_quote(r.project), url_quote(r.run_id), qs
         )
         return url
 
@@ -987,12 +1004,14 @@ class Run(object):
             return
 
         app_url = wandb.util.app_url(self._settings.base_url)
+        qs = self._get_url_query_string()
 
-        return "{base}/{entity}/{project}/sweeps/{sweepid}".format(
+        return "{base}/{entity}/{project}/sweeps/{sweepid}{qs}".format(
             base=app_url,
             entity=url_quote(r.entity),
             project=url_quote(r.project),
             sweepid=url_quote(sweep_id),
+            qs=qs,
         )
 
     def _get_run_name(self):
@@ -1586,9 +1605,7 @@ class Run(object):
 
         if isinstance(artifact_or_name, str):
             name = artifact_or_name
-            public_api = public.Api(
-                {"entity": r.entity, "project": r.project, "run": self.id}
-            )
+            public_api = self._public_api()
             artifact = public_api.artifact(type=type, name=name)
             if type is not None and type != artifact.type:
                 raise ValueError(
@@ -1669,9 +1686,34 @@ class Run(object):
         if isinstance(aliases, str):
             aliases = [aliases]
         artifact.finalize()
+        self._assert_can_log_artifact(artifact)
         self._backend.interface.publish_artifact(self, artifact, aliases)
         return artifact
 
+    def _assert_can_log_artifact(self, artifact):
+        if not self._settings._offline:
+            public_api = self._public_api()
+            expected_type = public.Artifact.expected_type(
+                public_api.client,
+                artifact.name,
+                public_api.settings["entity"],
+                public_api.settings["project"],
+            )
+            if expected_type is not None and artifact.type != expected_type:
+                raise ValueError(
+                    "Expected artifact type {}, got {}".format(
+                        expected_type, artifact.type
+                    )
+                )
+
+    def _public_api(self):
+        overrides = {"run": self.id}
+        run_obj = self._run_obj
+        if run_obj is not None:
+            overrides["entity"] = run_obj.entity
+            overrides["project"] = run_obj.project
+        return public.Api(overrides)
+
     def alert(self, title, text, level=None, wait_duration=None):
         """Launch an alert with the given title and text.
 
