@@ -1,4 +1,5 @@
 import base64
+import binascii
 import codecs
 import hashlib
 import os
@@ -13,6 +14,10 @@ def md5_string(string):
     return base64.b64encode(hash_md5.digest()).decode("ascii")
 
 
+def b64_string_to_hex(string):
+    return binascii.hexlify(base64.standard_b64decode(string)).decode("ascii")
+
+
 def md5_hash_file(path):
     hash_md5 = hashlib.md5()
     with open(path, "rb") as f:
