@@ -14,10 +14,10 @@ import six
 from six.moves import queue
 import wandb
 from wandb import util
+from wandb import wandb_lib
 from wandb import wandb_sdk
 from wandb.agents.pyagent import pyagent
 from wandb.apis import InternalApi
-from wandb.lib import config_util
 import yaml
 
 logger = logging.getLogger(__name__)
@@ -127,6 +127,7 @@ class Agent(object):
     KILL_DELAY = 30
     FLAPPING_MAX_SECONDS = 60
     FLAPPING_MAX_FAILURES = 3
+    MAX_INITIAL_FAILURES = 5
 
     def __init__(
         self, api, queue, sweep_id=None, function=None, in_jupyter=None, count=None
@@ -149,10 +150,16 @@ class Agent(object):
         self._failed = 0
         self._count = count
         self._sweep_command = []
+        self._max_initial_failures = wandb.env.get_agent_max_initial_failures(
+            self.MAX_INITIAL_FAILURES
+        )
         if self._report_interval is None:
             raise AgentError("Invalid agent report interval")
         if self._kill_delay is None:
             raise AgentError("Invalid agent kill delay")
+        # if the directory to log to is not set, set it
+        if os.environ.get("WANDB_DIR") is None:
+            os.environ["WANDB_DIR"] = os.path.abspath(os.getcwd())
 
     def is_flapping(self):
         """Flapping occurs if the agents receives FLAPPING_MAX_FAILURES non-0
@@ -162,6 +169,12 @@ class Agent(object):
         if time.time() < wandb.START_TIME + self.FLAPPING_MAX_SECONDS:
             return self._failed >= self.FLAPPING_MAX_FAILURES
 
+    def is_failing(self):
+        return (
+            self._failed >= self._finished
+            and self._max_initial_failures <= self._failed
+        )
+
     def run(self):  # noqa: C901
 
         # TODO: catch exceptions, handle errors, show validation warnings, and make more generic
@@ -217,6 +230,16 @@ class Agent(object):
                             )
                             self._running = False
                             break
+                        if self.is_failing():
+                            logger.error(
+                                "Detected %i failed runs in a row, shutting down.",
+                                self._max_initial_failures,
+                            )
+                            logger.info(
+                                "To change this value set WANDB_AGENT_MAX_INITIAL_FAILURES=val"
+                            )
+                            self._running = False
+                            break
                     logger.info("Cleaning up finished run: %s", run_id)
                     del self._run_processes[run_id]
                     self._last_report_time = None
@@ -330,9 +353,14 @@ class Agent(object):
         json_file = os.path.join(
             "wandb", "sweep-" + sweep_id, "config-" + run_id + ".json"
         )
-        config_util.save_config_file_from_dict(config_file, command["args"])
+
         os.environ[wandb.env.RUN_ID] = run_id
-        os.environ[wandb.env.CONFIG_PATHS] = config_file
+        os.environ[wandb.env.CONFIG_PATHS] = os.path.join(
+            os.environ.get(wandb.env.DIR), config_file
+        )
+        wandb_lib.config_util.save_config_file_from_dict(
+            os.environ[wandb.env.CONFIG_PATHS], command["args"]
+        )
 
         env = dict(os.environ)
 
@@ -496,12 +524,28 @@ def run_agent(
 def agent(sweep_id, function=None, entity=None, project=None, count=None):
     """Generic agent entrypoint, used for CLI or jupyter.
 
-    Args:
+    Will run a function or program with configuration parameters specified
+        by server.
+
+    Arguments:
         sweep_id (dict): Sweep ID generated by CLI or sweep API
-        function (func, optional): A function to call instead of the "program" specifed in the config
+        function (func, optional): A function to call instead of the "program"
+            specifed in the config.
         entity (str, optional): W&B Entity
         project (str, optional): W&B Project
         count (int, optional): the number of trials to run.
+
+    Examples:
+        Run a sample sweep over a function:
+        def train():
+            with wandb.init() as run:
+                print("config:", dict(run.config))
+                for epoch in range(35):
+                    print("running", epoch)
+                    wandb.log({"metric": run.config.param1, "epoch": epoch})
+                    time.sleep(1)
+
+        wandb.agent(sweep_id, function=train)
     """
     global _INSTANCES
     _INSTANCES += 1
@@ -510,7 +554,7 @@ def agent(sweep_id, function=None, entity=None, project=None, count=None):
         wandb_sdk.wandb_login._login(_silent=True)
         if function:
             return pyagent(sweep_id, function, entity, project, count)
-        in_jupyter = wandb._get_python_type() != "python"
+        in_jupyter = wandb.wandb_sdk.lib.ipython._get_python_type() != "python"
         return run_agent(
             sweep_id,
             function=function,
