@@ -26,19 +26,22 @@ def nested_shape(array_or_tuple, seen=None):
      """
     if seen is None:
         seen = set()
-    if hasattr(array_or_tuple, 'size'):
+    if hasattr(array_or_tuple, "size"):
         # pytorch tensors use V.size() to get size of tensor
         return list(array_or_tuple.size())
-    elif hasattr(array_or_tuple, 'get_shape'):
+    elif hasattr(array_or_tuple, "get_shape"):
         # tensorflow uses V.get_shape() to get size of tensor
         return array_or_tuple.get_shape().as_list()
-    elif hasattr(array_or_tuple, 'shape'):
+    elif hasattr(array_or_tuple, "shape"):
         return array_or_tuple.shape
 
     seen.add(id(array_or_tuple))
     try:
         # treat object as iterable
-        return [nested_shape(item, seen) if id(item) not in seen else 0 for item in list(array_or_tuple)]
+        return [
+            nested_shape(item, seen) if id(item) not in seen else 0
+            for item in list(array_or_tuple)
+        ]
     except TypeError:
         # object is not actually iterable
         # LB: Maybe we should throw an error?
@@ -79,7 +82,16 @@ class TorchHistory(object):
         self._is_cuda_histc_supported = None
         self._jupyter_run = None
 
-    def add_log_hooks_to_pytorch_module(self, module, name=None, prefix='', log_parameters=True, log_gradients=True, log_freq=0, jupyter_run=None):
+    def add_log_hooks_to_pytorch_module(
+        self,
+        module,
+        name=None,
+        prefix="",
+        log_parameters=True,
+        log_gradients=True,
+        log_freq=0,
+        jupyter_run=None,
+    ):
         """ This instuments hooks into the pytorch module
         log_parameters - log parameters after a forward pass
         log_gradients - log gradients after a backward pass
@@ -94,6 +106,7 @@ class TorchHistory(object):
         module._wandb_hook_names = []
 
         if log_parameters:
+
             def parameter_log_hook(module, input_, output, log_track):
                 if not log_track_update(log_track):
                     return
@@ -103,37 +116,44 @@ class TorchHistory(object):
                         data = parameter.data
                     else:
                         data = parameter
-                    self.log_tensor_stats(
-                        data.cpu(), 'parameters/' + prefix + name)
+                    self.log_tensor_stats(data.cpu(), "parameters/" + prefix + name)
+
             log_track_params = log_track_init(log_freq)
             hook = module.register_forward_hook(
-                lambda mod, inp, outp: parameter_log_hook(mod, inp, outp, log_track_params))
-            self._hook_handles['parameters/'+prefix] = hook
-            module._wandb_hook_names.append('parameters/'+prefix)
+                lambda mod, inp, outp: parameter_log_hook(
+                    mod, inp, outp, log_track_params
+                )
+            )
+            self._hook_handles["parameters/" + prefix] = hook
+            module._wandb_hook_names.append("parameters/" + prefix)
 
         if log_gradients:
             for name, parameter in module.named_parameters():
                 if parameter.requires_grad:
                     log_track_grad = log_track_init(log_freq)
-                    module._wandb_hook_names.append('gradients/' + prefix + name)
+                    module._wandb_hook_names.append("gradients/" + prefix + name)
                     self._hook_variable_gradient_stats(
-                        parameter, 'gradients/' + prefix + name, log_track_grad)
+                        parameter, "gradients/" + prefix + name, log_track_grad
+                    )
 
     def log_tensor_stats(self, tensor, name):
         """Add distribution statistics on a tensor's elements to the current History entry
         """
         # TODO Handle the case of duplicate names.
 
-        if (isinstance(tensor, tuple) or isinstance(tensor, list)):
-            while (isinstance(tensor, tuple) or isinstance(tensor, list)) and (isinstance(tensor[0], tuple) or isinstance(tensor[0], list)):
+        if isinstance(tensor, tuple) or isinstance(tensor, list):
+            while (isinstance(tensor, tuple) or isinstance(tensor, list)) and (
+                isinstance(tensor[0], tuple) or isinstance(tensor[0], list)
+            ):
                 tensor = [item for sublist in tensor for item in sublist]
             tensor = torch.cat([t.reshape(-1) for t in tensor])
 
         # checking for inheritance from _TensorBase didn't work for some reason
-        if not hasattr(tensor, 'shape'):
+        if not hasattr(tensor, "shape"):
             cls = type(tensor)
-            raise TypeError('Expected Tensor, not {}.{}'.format(
-                cls.__module__, cls.__name__))
+            raise TypeError(
+                "Expected Tensor, not {}.{}".format(cls.__module__, cls.__name__)
+            )
         history = self._history()
 
         # recover history from run if using jupyter
@@ -167,9 +187,7 @@ class TorchHistory(object):
         # For pytorch 0.3 we use unoptimized numpy histograms (detach is new in 0.4)
         if not hasattr(flat, "detach"):
             tensor = flat.cpu().clone().numpy()
-            history._row_update({
-                name: wandb.Histogram(tensor)
-            })
+            history._row_update({name: wandb.Histogram(tensor)})
             return
 
         if flat.is_cuda:
@@ -182,7 +200,7 @@ class TorchHistory(object):
                     check = flat.histc(bins=self._num_bins)
                 except RuntimeError as e:
                     # Only work around missing support with specific exception
-                    #if str(e).startswith("_th_histc is not implemented"):
+                    # if str(e).startswith("_th_histc is not implemented"):
                     #    self._is_cuda_histc_supported = False
                     # On second thought, 0.4.1 doesnt have support and maybe there are other issues
                     # lets disable more broadly for now
@@ -226,7 +244,7 @@ class TorchHistory(object):
             num_buckets = len(bins_np) - 1
             for i in range(num_buckets):
                 start = bins_np[i]
-                end = bins_np[i+1]
+                end = bins_np[i + 1]
                 # There are 3 cases to consider here, all of which mean we've found the right bucket
                 # 1. The bucket range contains zero.
                 # 2. The bucket range lower bound *is* zero.
@@ -239,10 +257,9 @@ class TorchHistory(object):
             tensor = torch.Tensor(tensor_np)
             bins = torch.Tensor(bins_np)
 
-        history._row_update({
-            name: wandb.Histogram(np_histogram=(
-                tensor.tolist(), bins.tolist()))
-        })
+        history._row_update(
+            {name: wandb.Histogram(np_histogram=(tensor.tolist(), bins.tolist()))}
+        )
 
     def _hook_variable_gradient_stats(self, var, name, log_track):
         """Logs a Variable's gradient's distribution statistics next time backward()
@@ -250,13 +267,15 @@ class TorchHistory(object):
         """
         if not isinstance(var, torch.autograd.Variable):
             cls = type(var)
-            raise TypeError('Expected torch.Variable, not {}.{}'.format(
-                cls.__module__, cls.__name__))
+            raise TypeError(
+                "Expected torch.Variable, not {}.{}".format(
+                    cls.__module__, cls.__name__
+                )
+            )
 
         handle = self._hook_handles.get(name)
         if handle is not None and self._torch_hook_handle_is_valid(handle):
-            raise ValueError(
-                'A hook has already been set under name "{}"'.format(name))
+            raise ValueError('A hook has already been set under name "{}"'.format(name))
 
         def _callback(grad, log_track):
             if not log_track_update(log_track):
@@ -285,7 +304,6 @@ class TorchHistory(object):
 
 
 class TorchGraph(wandb.data_types.Graph):
-
     def __init__(self):
         super(TorchGraph, self).__init__("torch")
 
@@ -304,8 +322,10 @@ class TorchGraph(wandb.data_types.Graph):
             modules.add(id(module))
             if not isinstance(output, tuple):
                 output = (output,)
-            parameters = [(pname, list(param.size()))
-                          for pname, param in module.named_parameters()]
+            parameters = [
+                (pname, list(param.size()))
+                for pname, param in module.named_parameters()
+            ]
 
             node = Node(
                 id=id(module),
@@ -313,18 +333,18 @@ class TorchGraph(wandb.data_types.Graph):
                 class_name=str(module),
                 output_shape=nested_shape(output),
                 parameters=parameters,
-                num_parameters=[reduce(mul, size, 1)
-                                for (pname, size) in parameters]
+                num_parameters=[reduce(mul, size, 1) for (pname, size) in parameters],
             )
             graph.nodes_by_id[id(module)] = node
             for param in module.parameters():
                 graph.nodes_by_id[id(param)] = node
             graph.add_node(node)
             if not graph.criterion_passed:
-                if hasattr(output[0], 'grad_fn'):
+                if hasattr(output[0], "grad_fn"):
                     graph.criterion = output[0].grad_fn
-                elif isinstance(output[0], list) and hasattr(output[0][0], 'grad_fn'):
+                elif isinstance(output[0], list) and hasattr(output[0][0], "grad_fn"):
                     graph.criterion = output[0][0].grad_fn
+
         return after_forward_hook
 
     def hook_torch_modules(self, module, criterion=None, prefix=None, graph_idx=0):
@@ -335,7 +355,8 @@ class TorchGraph(wandb.data_types.Graph):
         graph = self
         if hasattr(module, "_wandb_watch_called") and module._wandb_watch_called:
             raise ValueError(
-                "You can only call `wandb.watch` once per model.  Pass a new instance of the model if you need to call wandb.watch again in your code.")
+                "You can only call `wandb.watch` once per model.  Pass a new instance of the model if you need to call wandb.watch again in your code."
+            )
         module._wandb_watch_called = True
         if criterion:
             graph.criterion = criterion
@@ -355,13 +376,21 @@ class TorchGraph(wandb.data_types.Graph):
             # Torch 0.3   uses containers
             #       0.4   has ModuleList
             #       0.4.1 has ModuleDict
-            module_types = [getattr(torch.nn, module_classname)
-                            for module_classname in ("Container", "Sequential", "ModuleList", "ModuleDict")
-                            if hasattr(torch.nn, module_classname)]
+            module_types = [
+                getattr(torch.nn, module_classname)
+                for module_classname in (
+                    "Container",
+                    "Sequential",
+                    "ModuleList",
+                    "ModuleDict",
+                )
+                if hasattr(torch.nn, module_classname)
+            ]
 
             if isinstance(sub_module, tuple(module_types)):
                 self.hook_torch_modules(sub_module, prefix=name)
             else:
+
                 def backward_hook(module, input, output):
                     [hook.remove() for hook in hooks]
                     graph.loaded = True
@@ -369,35 +398,41 @@ class TorchGraph(wandb.data_types.Graph):
                         wandb.run.summary["graph_%i" % graph_idx] = graph
                     else:
                         wandb.termwarn(
-                            "wandb.watch was called without a call to wandb.init, call wandb.init before wandb.watch", repeat=False)
+                            "wandb.watch was called without a call to wandb.init, call wandb.init before wandb.watch",
+                            repeat=False,
+                        )
                     # TODO: Keeping this here as a starting point for adding graph data
                     if not graph.loaded:
+
                         def traverse(node, functions=[]):
-                            if hasattr(node, 'grad_fn'):
+                            if hasattr(node, "grad_fn"):
                                 node = node.grad_fn
 
-                            if hasattr(node, 'variable'):
+                            if hasattr(node, "variable"):
                                 node = graph.nodes_by_id.get(id(node.variable))
                                 if node:
                                     node.functions = list(functions)
                                     del functions[:]
 
-                            if hasattr(node, 'next_functions'):
+                            if hasattr(node, "next_functions"):
                                 functions.append(type(node).__name__)
                                 for f in node.next_functions:
                                     if f[0]:
                                         functions.append(type(f[0]).__name__)
                                         traverse(f[0], functions)
 
-                            if hasattr(node, 'saved_tensors'):
+                            if hasattr(node, "saved_tensors"):
                                 for t in node.saved_tensors:
                                     traverse(t)
+
                         traverse(graph.criterion)
 
                 hooks.append(
-                    sub_module.register_forward_hook(self.create_forward_hook(name, modules)))
-                hooks.append(
-                    sub_module.register_backward_hook(backward_hook))
+                    sub_module.register_forward_hook(
+                        self.create_forward_hook(name, modules)
+                    )
+                )
+                hooks.append(sub_module.register_backward_hook(backward_hook))
 
     @classmethod
     def from_torch_layers(cls, module_graph, variable):
@@ -423,7 +458,8 @@ class TorchGraph(wandb.data_types.Graph):
 
         module_nodes_by_hash = {id(n): n for n in module_graph.nodes}
         module_parameter_nodes = [
-            n for n in module_graph.nodes if isinstance(n.obj, torch.nn.Parameter)]
+            n for n in module_graph.nodes if isinstance(n.obj, torch.nn.Parameter)
+        ]
 
         names_by_pid = {id(n.obj): n.name for n in module_parameter_nodes}
 
@@ -452,7 +488,9 @@ class TorchGraph(wandb.data_types.Graph):
         node_depths = {id(n): d for n, d in module_graph[0].descendent_bfs()}
         parameter_module_names = {}
         parameter_modules = {}
-        for param_node in (n for n in module_graph.nodes if isinstance(n.obj, torch.nn.Parameter)):
+        for param_node in (
+            n for n in module_graph.nodes if isinstance(n.obj, torch.nn.Parameter)
+        ):
             pid = id(param_node.obj)
             best_node = None
             best_depth = None
@@ -464,7 +502,10 @@ class TorchGraph(wandb.data_types.Graph):
                 reachable_params = module_reachable_params[id(module)]
                 if pid in reachable_params:
                     depth = node_depths[id(node)]
-                    if best_node is None or (len(reachable_params), depth) <= (len(best_reachable_params), best_depth):
+                    if best_node is None or (len(reachable_params), depth) <= (
+                        len(best_reachable_params),
+                        best_depth,
+                    ):
                         best_node = node
                         best_depth = depth
                         best_reachable_params = reachable_params
@@ -483,8 +524,7 @@ class TorchGraph(wandb.data_types.Graph):
 
         module_nodes_by_pid = {id(n.obj): n for n in module_graph.nodes}
 
-        compute_graph, compute_node_vars = cls.from_torch_compute_graph(
-            variable)
+        compute_graph, compute_node_vars = cls.from_torch_compute_graph(variable)
         for node, _ in reversed(list(compute_graph[0].ancestor_bfs())):
             param = compute_node_vars.get(node.id)
             pid = id(param)
@@ -500,7 +540,8 @@ class TorchGraph(wandb.data_types.Graph):
                 rmg_module = rmg_nodes_by_pid[mid]
             else:
                 rmg_module = rmg_nodes_by_pid[mid] = Node(
-                    id=next(rmg_ids), node=module_nodes_by_pid[mid])
+                    id=next(rmg_ids), node=module_nodes_by_pid[mid]
+                )
                 reduced_module_graph.add_node(rmg_module)
                 reduced_module_graph.add_edge(rmg_root, rmg_module)
 
